/*------------------------------------------------*/
/* FFTEST : A test program for FFT module         */

#include <string.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "ffft.h"		/* Defs for using Fixed-point FFT module */
#include "mydefs.h"

/*------------------------------------------------*/
/* Global variables                               */

int16_t capture[FFT_N];
complex_t bfly_buff[FFT_N];
uint16_t spektrum[FFT_N/2];
uint16_t pwm_setting[CHANNELS];
uint8_t counter[CHANNELS];
volatile uint8_t peak[CHANNELS];
const uint16_t pwmtable_10[64]  PROGMEM = { 0, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 4, 4, 5,
											5, 6, 6, 7, 8, 9, 10, 11, 12, 13, 15, 17,
											19, 21, 23, 26, 29, 32, 36, 40, 44, 49, 55,
											61, 68, 76, 85, 94, 105, 117, 131, 146, 162,
											181, 202, 225, 250, 279, 311, 346, 386, 430,
											479, 534, 595, 663, 739, 824, 918, 1023 };

/*------------------------------------------------*/
/* Timer 1 Output COMPARE A Interrupt             */
 
ISR( TIMER1_COMPA_vect ) {
	uint8_t tmp=0;
	static uint16_t pwm_cnt = 0;
	///////////////////////////////////////////////
	//Hier CLKR und UpDown setzten (CS ist permanent auf GND)
	//AD5220 schaltet bei fallender Flanke von CLKR. RUpDown muss 10ns vorher gesetzt sein
	//                   __
	// CLKR        _____/  \__
	// RUpDown           _____    
	//                  /           
	//                R vergrssern
	//
	//                   __
	// CLKR        _____/  \__
	// RUpDown         _     
	//                  \_____          
	//                R verkleinern 
    //////////////////////////////////////////////
  OCR1A += (uint16_t) T_PWM;

	if(pwm_setting[0] > pwm_cnt)	tmp |= _BV(LED1);
	if(pwm_setting[1] > pwm_cnt)	tmp |= _BV(LED2);
	if(pwm_setting[2] > pwm_cnt)	tmp |= _BV(LED3);

	LED_PORT = tmp;

	if(++pwm_cnt == PWM_STEPS)
        pwm_cnt = 0;

	// mu aus Performancegrnden leider mit in die ISR
	if((pwm_cnt == 512) || (pwm_cnt == 0)) {
		for(uint8_t y = 0; y < CHANNELS; y++) {
			if(counter[y] == 0) 
			if(peak[y]) peak[y]--;		
			}			

	}
}

/*------------------------------------------------*/
/* Capture waveform                               */

void capture_wave(int16_t *buffer, uint16_t count) {
	ADMUX = _BV(REFS0)|_BV(ADLAR)|_BV(MUX2)|_BV(MUX0);				// channel

	do {

		ADCSRA = _BV(ADEN)|_BV(ADSC)|_BV(ADATE)|_BV(ADIF)|_BV(ADPS2)|_BV(ADPS1);

		while(bit_is_clear(ADCSRA, ADIF));
		*buffer++ = ADC + INT16_MIN;
	} while(--count);

	ADCSRA = 0;
}

/*------------------------------------------------*/
/*                                                */

int main (void) {

	uint16_t s;
	LED_DIR  |= _BV(LED1)|_BV(LED2)|_BV(LED3)|_BV(CLKR)|_BV(RUpDown);
	LED_PORT &= ~(_BV(LED1)|_BV(LED2)|_BV(LED3)|_BV(CLKR)|_BV(RUpDown));
	TCCR1B = _BV(CS10);											                    // Timer luft mit vollem Systemtakt
	TIMSK |= _BV(OCIE1A);											                  // Interrupt freischalten

	sei();



	for(;;) {
		capture_wave(capture, FFT_N);
		fft_input(capture, bfly_buff);
		fft_execute(bfly_buff);
		fft_output(bfly_buff, spektrum);		
		// Array in bentigter Gre vor Wiederverwendung 'subern'
		memset(capture, 0, 12);

		// Bnder durch Ausprobieren gefunden
		for (uint8_t n = 0; n < FFT_N / 2; n++) {
			s = spektrum[n];
			
			if((n >= 0)  && (n <= 0)) {
				if((capture[0] + s) < SCORE_MAX) capture[0] += s;
				else capture[0] = SCORE_MAX;
			}

			if((n >= 15)  && (n <= 20
			)) {
				if((capture[1] + s) < SCORE_MAX) capture[1] += s;
				else capture[1] = SCORE_MAX;
			}

			if((n >= 40) && (n <= 63)) {
				if((capture[2] + s) < SCORE_MAX) capture[2] += s;
				else capture[2] = SCORE_MAX;
			}


		}

		for(uint8_t y = 0; y < CHANNELS; y++) {
			if(capture[y] > THRESHOLD) capture[y] -= THRESHOLD;		// Schwellwert abziehen
			else capture[y] = 0;
			capture[y] /= 256;										// Ergebnis teilen
			if(capture[y] > 63) capture[y] = 63;			        // auf 64 Stufen begrenzen (Murphy's Law)
			if(counter[y]) counter[y]= counter[y] - 1;				// vor dem zurckfaden der LED's etwas warten
			if(capture[y] >= peak[y]) {								// so 'ne Art PeakHold
				peak[y] = capture[y];
				counter[y] = 2;							
			}	
			cli();
			pwm_setting[y] = pgm_read_word(&pwmtable_10[peak[y]]);	// Helligkeitsstufe zuweisen
			sei();
		}
	}
}
